Index: common/common.c
===================================================================
--- common/common.c	(revision 654)
+++ common/common.c	(working copy)
@@ -63,6 +63,7 @@
     param->i_fps_num       = 25;
     param->i_fps_den       = 1;
     param->i_level_idc     = 51; /* as close to "unrestricted" as we can get */
+    param->i_slice_max     = 0; 
 
     /* Encoder parameters */
     param->i_frame_reference = 1;
Index: common/common.h
===================================================================
--- common/common.h	(revision 654)
+++ common/common.h	(working copy)
@@ -127,7 +127,7 @@
 #define X264_BFRAME_MAX 16
 #define X264_THREAD_MAX 128
 #define X264_SLICE_MAX 4
-#define X264_NAL_MAX (4 + X264_SLICE_MAX)
+#define X264_NAL_MAX (32 + X264_SLICE_MAX)
 #define X264_THREAD_HEIGHT 24 // number of pixels (per thread) in progress at any given time. could theoretically be as low as 22
 
 /****************************************************************************
@@ -312,6 +312,7 @@
     struct
     {
         int         i_nal;
+        int         i_nal_bufsize;
         x264_nal_t  nal[X264_NAL_MAX];
         int         i_bitstream;    /* size of p_bitstream */
         uint8_t     *p_bitstream;   /* will hold data for all nal */
Index: encoder/encoder.c
===================================================================
--- encoder/encoder.c	(revision 654)
+++ encoder/encoder.c	(working copy)
@@ -1021,6 +1021,7 @@
     int i_skip;
     int mb_xy;
     int i;
+    int i_pos_max;
 
     /* init stats */
     memset( &h->stat.frame, 0, sizeof(h->stat.frame) );
@@ -1042,6 +1043,8 @@
     h->mb.i_last_qp = h->sh.i_qp;
     h->mb.i_last_dqp = 0;
 
+    i_pos_max = bs_pos(&h->out.bs) + (h->param.i_slice_max<<3);
+
     for( mb_xy = h->sh.i_first_mb, i_skip = 0; mb_xy < h->sh.i_last_mb; )
     {
         const int i_mb_y = mb_xy / h->sps->i_mb_width;
@@ -1145,37 +1148,64 @@
         }
         else
             mb_xy++;
-    }
 
-    if( h->param.b_cabac )
-    {
-        /* end of slice */
-        x264_cabac_encode_terminal( &h->cabac, 1 );
-    }
-    else if( i_skip > 0 )
-    {
-        bs_write_ue( &h->out.bs, i_skip );  /* last skip run */
-    }
+        if ((mb_xy >= h->sh.i_last_mb)
+            || ((h->param.i_slice_max > 0) && (bs_pos(&h->out.bs ) > i_pos_max))) //(i_mb_x % 32 == 0) && 
+        {
+            if( h->param.b_cabac )
+            {
+            /* end of slice */
+                x264_cabac_encode_terminal( &h->cabac, 1 );
+            }
+            else if( i_skip > 0 )
+            {
+                bs_write_ue( &h->out.bs, i_skip );  /* last skip run */
+            }
 
-    if( h->param.b_cabac )
-    {
-        x264_cabac_encode_flush( &h->cabac );
-        h->out.bs.p = h->cabac.p;
-    }
-    else
-    {
-        /* rbsp_slice_trailing_bits */
-        bs_rbsp_trailing( &h->out.bs );
-    }
+            if( h->param.b_cabac )
+            {
+                x264_cabac_encode_flush( &h->cabac );
+                h->out.bs.p = h->cabac.p;
+            }
+            else
+            {
+                /* rbsp_slice_trailing_bits */
+                bs_rbsp_trailing( &h->out.bs );
+            }
 
-    x264_nal_end( h );
+            x264_nal_end( h );
 
-    /* Compute misc bits */
-    h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
-                              + NALU_OVERHEAD * 8
-                              - h->stat.frame.i_itex_bits
-                              - h->stat.frame.i_ptex_bits
-                              - h->stat.frame.i_hdr_bits;
+            if (mb_xy >= h->sh.i_last_mb)
+            {
+                /* Compute misc bits */
+                h->stat.frame.i_misc_bits = bs_pos( &h->out.bs )
+                            + NALU_OVERHEAD * 8
+                            - h->stat.frame.i_itex_bits
+                            - h->stat.frame.i_ptex_bits
+                            - h->stat.frame.i_hdr_bits;
+                return;
+            }
+
+            h->sh.i_first_mb = mb_xy;
+
+            x264_nal_start( h, h->i_nal_type, h->i_nal_ref_idc );
+            x264_slice_header_write( &h->out.bs, &h->sh, h->i_nal_ref_idc );
+
+            if( h->param.b_cabac )
+            {
+                /* alignment needed */
+                bs_align_1( &h->out.bs );
+
+                /* init cabac */
+                x264_cabac_context_init( &h->cabac, h->sh.i_type, h->sh.i_qp, h->sh.i_cabac_init_idc );
+                x264_cabac_encode_init ( &h->cabac, h->out.bs.p, h->out.bs.p_end );
+            }
+            h->mb.i_last_qp = h->sh.i_qp;
+            h->mb.i_last_dqp = 0;
+
+            i_pos_max = bs_pos(&h->out.bs) + (h->param.i_slice_max<<3);
+        }
+    }
 }
 
 static void x264_thread_sync_context( x264_t *dst, x264_t *src )
@@ -1207,16 +1237,15 @@
 
 static int x264_slices_write( x264_t *h )
 {
-    int i_frame_size;
-
+    int i;
+    int i_frame_size = 0;
+    int startnal = h->out.i_nal;
 #if VISUALIZE
     if( h->param.b_visualize )
         x264_visualize_init( h );
 #endif
 
     x264_stack_align( x264_slice_write, h );
-    i_frame_size = h->out.nal[h->out.i_nal-1].i_payload;
-    x264_fdec_filter_row( h, h->sps->i_mb_height );
 
 #if VISUALIZE
     if( h->param.b_visualize )
@@ -1226,6 +1255,11 @@
     }
 #endif
 
+    for (i=startnal; i<h->out.i_nal; i++)
+    {
+        i_frame_size += h->out.nal[i].i_payload;
+    }
+
     h->out.i_frame_size = i_frame_size;
     return 0;
 }
Index: x264.c
===================================================================
--- x264.c	(revision 654)
+++ x264.c	(working copy)
@@ -180,6 +180,8 @@
     H1( "      --no-deblock            Disable loop filter\n" );
     H0( "  -f, --deblock <alpha:beta>  Loop filter AlphaC0 and Beta parameters [%d:%d]\n",
                                        defaults->i_deblocking_filter_alphac0, defaults->i_deblocking_filter_beta );
+    H0( "      --max-slicesize <integer>  Limit the size of each slice (in bytes) (0 => unlimited)\n");
+
     H0( "      --interlaced            Enable pure-interlaced mode\n" );
     H0( "\n" );
     H0( "Ratecontrol:\n" );
@@ -365,6 +367,7 @@
 #define OPT_PROGRESS 261
 #define OPT_VISUALIZE 262
 #define OPT_LONGHELP 263
+#define OPT_SLICE_MAX 264
 
         static struct option long_options[] =
         {
@@ -435,6 +438,7 @@
             { "zones",   required_argument, NULL, 0 },
             { "qpfile",  required_argument, NULL, OPT_QPFILE },
             { "threads", required_argument, NULL, 0 },
+            { "max-slicesize", required_argument, NULL, OPT_SLICE_MAX },
             { "thread-input", no_argument,  NULL, OPT_THREAD_INPUT },
             { "non-deterministic", no_argument, NULL, 0 },
             { "no-psnr", no_argument,       NULL, 0 },
@@ -537,6 +541,9 @@
                 param->i_scenecut_threshold = -1;
                 param->b_bframe_adaptive = 0;
                 break;
+            case OPT_SLICE_MAX:
+                param->i_slice_max = atoi(optarg);
+                break;
             case OPT_THREAD_INPUT:
                 b_thread_input = 1;
                 break;
Index: x264.h
===================================================================
--- x264.h	(revision 654)
+++ x264.h	(working copy)
@@ -271,6 +271,8 @@
     int b_aud;                  /* generate access unit delimiters */
     int b_repeat_headers;       /* put SPS/PPS before each keyframe */
     int i_sps_id;               /* SPS and PPS id number */
+    int i_slice_max; /* maximum slice size in bytes, split slices if larger than this many bytes (0 => unlimited)  */
+
 } x264_param_t;
 
 typedef struct {